/***************************************************************************
 * Artekit_SI7021.c
 * Copyright (C) 2014 Artekit Italy
 * http://www.artekit.eu
 *
 * This class provides support for the AK-SI7021 humidity and temperature
 * sensor, to be used with Arduino.
 * This class depends on the Wire.h library, that must be included in
 * the main sketch.
 *
 * Product page:
 * http://www.artekit.eu/products/sensors/ak-si7021/
 *
***************************************************************************/

#include "Artekit_SI7021.h"
#include <Wire.h>

Artekit_SI7021::Artekit_SI7021()
{
}

Artekit_SI7021::~Artekit_SI7021()
{
}
    
float Artekit_SI7021::ReadTemperature(bool bFromHRSample)
{
    uint8_t cmd = (bFromHRSample) ? 0xE0 : 0xE3;
    signed short val = Measure(cmd);
    return (  ((175.72f * val) / 65536) - 46.85f);
}


float Artekit_SI7021::ReadHumidity(void)
{
    unsigned short val = Measure(0xE5);
    return ( (float) ((125 * (float) val) / 65536) - 6);
}

uint8_t Artekit_SI7021::ReadUserRegister(void)
{
    Wire.beginTransmission(SI7021_ADDR);
    Wire.write(0xE7);
    Wire.endTransmission(false);
    Wire.requestFrom(SI7021_ADDR, 1);
    return Wire.read();
}

void Artekit_SI7021::WriteUserRegister(uint8_t value)
{
    Wire.beginTransmission(SI7021_ADDR);
    Wire.write(0xE6);
    Wire.write(value);
    Wire.endTransmission(true);
}

void Artekit_SI7021::EnableHeater(void)
{
    uint8_t reg = ReadUserRegister();
    reg |= (1 << 2);
    WriteUserRegister(reg);
}

void Artekit_SI7021::DisableHeater(void)
{
    uint8_t reg = ReadUserRegister();
    reg &= ~(1 << 2);
    WriteUserRegister(reg);
}

uint16_t Artekit_SI7021::Measure(uint8_t cmd)
{
    uint16_t val = 0;
    Wire.beginTransmission(SI7021_ADDR);
    Wire.write(cmd);

    Wire.endTransmission(false);

    Wire.requestFrom(SI7021_ADDR, 2);
    val = Wire.read() << 8;
    val |= Wire.read() & 0x0FF;

    return val;
}
